/*
 * Copyright 2000 Perforce Software.  All rights reserved.
 *
 * This file is part of Perforce - the FAST SCM System.
 *
 * Author: Michael Bishop
 *
 * Tabs: 4 spaces
 *
 * This is just JNI glue to make the java calls go through to the
 * native StrDict object.
 *
 * The philosophy behind the Java glue is that, all of the native
 * objects are created natively. The pointer to that object is
 * passed to java where it stores it as an int. The value of that
 * int is actually a pointer in memory to the native object.
 * In this way, java can pass that int back to C code where it can
 * be casted to a C object pointer. When the java object is garbage
 * collected, it calls native code to delete the native pointer.
 *
 * All methods that are exposed with a java call are simply passed
 * through to native code where the native call is made.
 *
 */


#include "strdict_jni.h"
#include "stdhdrs.h"
#include "strbuf.h"
#include "strdict.h"

/*
 * Class:     com_perforce_client_api_StrDict
 * Method:    nDeleteInstance
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_com_perforce_client_api_StrDict_nDeleteInstance
	(JNIEnv *, jclass, jlong jRef)
{
	delete (StrDict *)jRef;
}


/*
 * Class:     com_perforce_client_api_StrDict
 * Method:    nSetVar
 * Signature: (JLjava/lang/String;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_com_perforce_client_api_StrDict_nSetVar
	(JNIEnv * env, jclass, jlong jRef, jstring jName, jstring jValue)
{
	StrDict * ref = (StrDict *)jRef;
	
	const char * cName = env->GetStringUTFChars(jName, NULL);
	const char * cValue = env->GetStringUTFChars(jValue, NULL);
	
	ref->SetVar( cName, cValue );
	
	env->ReleaseStringUTFChars(jName, cName);
	env->ReleaseStringUTFChars(jValue, cValue);
}

/*
 * Class:     com_perforce_client_api_StrDict
 * Method:    nGetVar
 * Signature: (JLjava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_perforce_client_api_StrDict_nGetVar
	(JNIEnv * env, jclass, jlong jRef, jstring jName)
{
	StrDict * ref = (StrDict *)jRef;
	
	const char * cName = env->GetStringUTFChars(jName, NULL);
	char * cValue = ref->GetVar( cName )->Text();
	env->ReleaseStringUTFChars(jName, cName);
	
	jstring jValue = env->NewStringUTF(cValue);
	
	return jValue;
}

/*
 * Class:     com_perforce_client_api_StrDict
 * Method:    nGetName
 * Signature: (JI)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_perforce_client_api_StrDict_nGetName
	(JNIEnv * env, jclass, jlong jRef, jint index)
{
	StrDict * ref = (StrDict *)jRef;
	
	//
	// Because when we have an index, we have to get
	// both the name and a value (even though we are interested
	// only in the name)
	//
	StrRef name;
	StrRef value;
	
	if (ref->GetVar(index, name, value))
		// Return only the name
		return env->NewStringUTF(name.Text());
	else
		return NULL;
}

/*
 * Class:     com_perforce_client_api_StrDict
 * Method:    nRemoveVar
 * Signature: (JLjava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_com_perforce_client_api_StrDict_nRemoveVar
  (JNIEnv *env, jclass, jlong jRef, jstring jName)
{
	StrDict * ref = (StrDict *)jRef;
	
	const char * cName = env->GetStringUTFChars(jName, NULL);
	ref->RemoveVar(cName);
	env->ReleaseStringUTFChars(jName, cName);
}